# -*- coding: utf-8 -*-
# pyemtmad, EMT API wrapper - https://github.com/rmed/pyemtmad
# Copyright (C) 2016  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""This file contains the endpoints for bus services.

See http://opendata.emtmadrid.es/Servicios-web/BUS
"""

from pyemtmad import types as emtype
from pyemtmad import util

class BusApi(object):
    """Metaclass that contains the API methods for the bus endpoints."""

    def __init__(self, wrapper):
        """Initialization of the API module.

        Args:
            wrapper (Wrapper): Object that performs the requests to endpoints.
        """
        self._wrapper = wrapper
        self.make_request = self._wrapper.request_openbus

    def get_calendar(self, **kwargs):
        """Obtain EMT calendar for a range of dates.

        Args:
            start_day (int): Starting day of the month in format DD.
                The number is automatically padded if it only has one digit.
            start_month (int): Starting month number in format MM.
                The number is automatically padded if it only has one digit.
            start_year (int): Starting year number in format YYYY.
            end_day (int): Ending day of the month in format DD.
                The number is automatically padded if it only has one digit.
            end_month (int): Ending month number in format MM.
                The number is automatically padded if it only has one digit.
            end_year (int): Ending year number in format YYYY.

        Returns:
            Status boolean and parsed response (list[CalendarItem]), or message
            string in case of error.
        """
        # Endpoint parameters
        start_date = util.date_string(
            kwargs.get('start_day', '01'),
            kwargs.get('start_month', '01'),
            kwargs.get('start_year', '1970')
        )

        end_date = util.date_string(
            kwargs.get('end_day', '01'),
            kwargs.get('end_month', '01'),
            kwargs.get('end_year', '1970')
        )

        params = {'SelectDateBegin': start_date, 'SelectDateEnd': end_date}

        # Request
        result = self.make_request('bus', 'get_calendar', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.CalendarItem(**a) for a in values]

    def get_groups(self, **kwargs):
        """Obtain line types and details.

        Args:
            lang(str):  Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[BusGroupItem]), or message
            string in case of error.
        """
        # Endpoint parameters
        params = {'cultureInfo': util.language_code(kwargs.get('lang'))}

        # Request
        result = self.make_request('bus', 'get_groups', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.BusGroupItem(**a) for a in values]

    def get_list_lines(self, **kwargs):
        """Obtain lines with description and group.

        Args:
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.

        Returns:
            Status boolean and parsed response (list[ListLineInfo]), or message
            string in case of error.
        """
        # Endpoint parameters
        select_date = util.date_string(
            kwargs.get('day', '01'),
            kwargs.get('month', '01'),
            kwargs.get('year', '1970')
        )

        # Lines separated by |
        lines = util.ints_to_string(kwargs.get('lines', []))

        params = {'SelectDate': select_date, 'Lines': lines}

        # Request
        result = self.make_request('bus', 'get_list_lines', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.ListLineInfo(**a) for a in values]

    def get_nodes_lines(self, **kwargs):
        """Obtain stop IDs, coordinates and line information.

        Args:
            nodes (list[int] | int): nodes to query, may be empty to get
                all nodes.

        Returns:
            Status boolean and parsed response (list[NodeLinesItem]), or message
            string in case of error.
        """
        # Endpoint parameters
        params = {'Nodes': util.ints_to_string(kwargs.get('nodes', []))}

        # Request
        result = self.make_request('bus', 'get_nodes_lines', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.NodeLinesItem(**a) for a in values]

    def get_route_lines(self, **kwargs):
        """Obtain itinerary for one or more lines in the given date.

        Args:
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.

        Returns:
            Status boolean and parsed response (list[RouteLinesItem]), or message
            string in case of error.
        """
        # Endpoint parameters
        select_date = util.date_string(
            kwargs.get('day', '01'),
            kwargs.get('month', '01'),
            kwargs.get('year', '1970')
        )

        lines = util.ints_to_string(kwargs.get('lines', []))

        params = {'SelectDate': select_date, 'Lines': lines}

        # Request
        result = self.make_request('bus', 'get_route_lines', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.RouteLinesItem(**a) for a in values]

    def get_times_lines(self, **kwargs):
        """Obtain current line times for the given lines.

        Args:
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.

        Returns:
            Status boolean and parsed response (list[TimesLinesItem]), or message
            string in case of error.
        """
        # Endpoint parameters
        select_date = util.date_string(
            kwargs.get('day', '01'),
            kwargs.get('month', '01'),
            kwargs.get('year', '1970')
        )

        lines = util.ints_to_string(kwargs.get('lines', []))

        params = {'SelectDate': select_date, 'Lines': lines}

        # Request
        result = self.make_request('bus', 'get_times_lines', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.TimesLinesItem(**a) for a in values]

    def get_timetable_lines(self, **kwargs):
        """Obtain information on lines for a travel.

        Args:
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.

        Returns:
            Status boolean and parsed response (list[TimetableLinesItem]),
            or message string in case of error.
        """
        # Endpoint parameters
        select_date = util.date_string(
            kwargs.get('day', '01'),
            kwargs.get('month', '01'),
            kwargs.get('year', '1970')
        )

        lines = util.ints_to_string(kwargs.get('lines', []))

        # Request
        params = {'SelectDate': select_date, 'Lines': lines}
        result = self.make_request('bus', 'get_timetable_lines', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.TimetableLinesItem(**a) for a in values]



# -*- coding: utf-8 -*-
# pyemtmad, EMT API wrapper - https://github.com/rmed/pyemtmad
# Copyright (C) 2016  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""This file contains the endpoints for parking services.

See https://servicios.emtmadrid.es:8443/InfoParking/InfoParking.svc/json/help
"""

from pyemtmad import types as emtype
from pyemtmad import util

class ParkingApi(object):
    """Metaclass that contains the API methods for the parking endpoints."""

    def __init__(self, wrapper):
        """Initialization of the API module.

        Args:
            wrapper (Wrapper): Object that performs the requests to endpoints.
        """
        self._wrapper = wrapper
        self.make_request = self._wrapper.request_parking

    def detail_parking(self, **kwargs):
        """Obtain detailed info of a given parking.

        Args:
            lang (str):  Language code (*es* or *en*).
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            hour (int): Hour of the day in format hh.
                The number is automatically padded if it only has one digit.
            minute (int): Minute of the hour in format mm.
                The number is automatically padded if it only has one digit.
            parking (int): ID of the parking to query.
            family (str): Family code of the parking (3 chars).

        Returns:
            Status boolean and parsed response (list[ParkingDetails]), or message
            string in case of error.
        """
        # Endpoint parameters
        date = util.datetime_string(
            kwargs.get('day', 1),
            kwargs.get('month', 1),
            kwargs.get('year', 1970),
            kwargs.get('hour', 0),
            kwargs.get('minute', 0)
        )

        params = {
            'language': util.language_code(kwargs.get('lang')),
            'publicData': True,
            'date': date,
            'id': kwargs.get('parking'),
            'family': kwargs.get('family')
        }

        # Request
        result = self.make_request('detail_parking', {}, **params)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.ParkingDetails(**a) for a in values]

    def detail_poi(self, **kwargs):
        """Obtain detailed info of a given POI.

        Args:
            family (str): Family code of the POI (3 chars).
            lang (str): Language code (*es* or *en*).
            id (int): Optional, ID of the POI to query. Passing value -1 will
                result in information from all POIs.

        Returns:
            Status boolean and parsed response (list[PoiDetails]), or
            message string in case of error.
        """
        # Endpoint parameters
        params = {
            'language': util.language_code(kwargs.get('lang')),
            'family': kwargs.get('family')
        }

        if kwargs.get('id'):
            params['id'] = kwargs['id']

        # Request
        result = self.make_request('detail_poi', {}, **params)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.PoiDetails(**a) for a in values]

    def icon_description(self, **kwargs):
        """Obtain a list of elements that have an associated icon.

        Args:
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[IconDescription]), or
            message string in case of error.
        """
        # Endpoint parameters
        params = {'language': util.language_code(kwargs.get('lang'))}

        # Request
        result = self.make_request('icon_description', {}, **params)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.IconDescription(**a) for a in values]

    def info_parking_poi(self, **kwargs):
        """Obtain generic information on POIs and parkings.

        This returns a list of elements in a given radius from the coordinates.

        Args:
            radius (int): Radius of the search (in meters).
            latitude (double): Latitude in decimal degrees.
            longitude (double): Longitude in decimal degrees.
            lang (str): Language code (*es* or *en*).
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            hour (int): Hour of the day in format hh.
                The number is automatically padded if it only has one digit.
            minute (int): Minute of the hour in format mm.
                The number is automatically padded if it only has one digit.
            poi_info (list[tuple]): List of tuples with the format
                ``(list[family], type, category)`` to query. Check the API
                documentation.
            min_free (list[int]): Number of free spaces to check. Must have the
                same length of ``poi_info``.
            field_codes (list[tuple]): List of tuples with the format
                ``(list[codes], name)``. Check the API documentation.

        Returns:
            Status boolean and parsed response (list[InfoParkingPoi]), or
            message string in case of error.
        """
        # Endpoint parameters
        date = util.datetime_string(
            kwargs.get('day', 1),
            kwargs.get('month', 1),
            kwargs.get('year', 1970),
            kwargs.get('hour', 0),
            kwargs.get('minute', 0)
        )

        family_categories = []
        for element in kwargs.get('poi_info', []):
            family_categories.append({
                'poiCategory': {
                    'lstCategoryTypes': element[0]
                    },
                'poiFamily': element[1],
                'poiType': element[2]
            })

        field_codes = []
        for element in kwargs.get('field_codes', []):
            field_codes.append({
                'codes': {
                    'lstCodes': element[0]
                    },
                'nameField': element[1]
            })

        params = {
            'TFamilyTTypeTCategory': {
                'lstFamilyTypeCategory': family_categories
            },
            'coordinate': {
                'latitude': str(kwargs.get('latitude', '0.0')),
                'longitude': str(kwargs.get('longitude', '0.0'))
            },
            'dateTimeUse': date,
            'language': util.language_code(kwargs.get('lang')),
            'minimumPlacesAvailable': {
                'lstminimumPlacesAvailable': kwargs.get('min_free', [])
            },
            'nameFieldCodes': {
                'lstNameFieldCodes': field_codes
            },
            'radius': str(kwargs.get('radius', '0'))
        }

        # Request
        result = self.make_request('info_parking_poi', {}, **params)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.InfoParkingPoi(**a) for a in values]

    def list_features(self, **kwargs):
        """Obtain a list of parkings.

        Args:
            lang (str):  Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Parking]), or message
            string in case of error.
        """
        # Endpoint parameters
        params = {
            'language': util.language_code(kwargs.get('lang')),
            'publicData': True
        }

        # Request
        result = self.make_request('list_features', {}, **params)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.ParkingFeature(**a) for a in values]

    def list_parking(self, **kwargs):
        """Obtain a list of parkings.

        Args:
            lang (str):  Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Parking]), or message
            string in case of error.
        """
        # Endpoint parameters
        url_args = {'lang': util.language_code(kwargs.get('lang'))}

        # Request
        result = self.make_request('list_parking', url_args)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.Parking(**a) for a in values]

    def list_street_poi_parking(self, **kwargs):
        """Obtain a list of addresses and POIs.

        This endpoint uses an address to perform the search

        Args:
            lang (str): Language code (*es* or *en*).
            address (str): Address in which to perform the search.

        Returns:
            Status boolean and parsed response (list[ParkingPoi]), or message
            string in case of error.
        """
        # Endpoint parameters
        url_args = {
            'language': util.language_code(kwargs.get('lang')),
            'address': kwargs.get('address', '')
        }

        # Request
        result = self.make_request('list_street_poi_parking', url_args)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.ParkingPoi(**a) for a in values]

    def list_types_poi(self, **kwargs):
        """Obtain a list of families, types and categories of POI.

        Args:
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[ParkingPoiType]), or message
            string in case of error.
        """
        # Endpoint parameters
        url_args = {'language': util.language_code(kwargs.get('lang'))}

        # Request
        result = self.make_request('list_poi_types', url_args)

        if not util.check_result(result):
            return False, result.get('message', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'Data')
        return True, [emtype.ParkingPoiType(**a) for a in values]



# -*- coding: utf-8 -*-
# pyemtmad, EMT API wrapper - https://github.com/rmed/pyemtmad
# Copyright (C) 2016  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""This file contains the endpoints for geo services.

See http://opendata.emtmadrid.es/Servicios-web/GEO
"""

from pyemtmad import types as emtype
from pyemtmad import util

class GeoApi(object):
    """Metaclass that contains the API methods for the geo endpoints."""

    def __init__(self, wrapper):
        """Initialization of the API module.

        Args:
            wrapper (Wrapper): Object that performs the requests to endpoints.
        """
        self._wrapper = wrapper
        self.make_request = self._wrapper.request_openbus

    def get_arrive_stop(self, **kwargs):
        """Obtain bus arrival info in target stop.

        Args:
            stop_number (int): Stop number to query.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Arrival]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'idStop': kwargs.get('stop_number'),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_arrive_stop', **params)

        # Funny endpoint, no status code
        if not util.check_result(result, 'arrives'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'arrives')
        return True, [emtype.Arrival(**a) for a in values]

    def get_groups(self, **kwargs):
        """Obtain line types and details.

        Args:
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[GeoGroupItem]), or message
            string in case of error.
        """
        # Endpoint parameters
        params = {
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_groups', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.GeoGroupItem(**a) for a in values]

    def get_info_line(self, **kwargs):
        """Obtain basic information on a bus line on a given date.

        Args:
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Line]), or message string
            in case of error.
        """
        # Endpoint parameters
        select_date = '%02d/%02d/%d' % (
            kwargs.get('day', '01'),
            kwargs.get('month', '01'),
            kwargs.get('year', '1970')
        )

        params = {
            'fecha': select_date,
            'line': util.ints_to_string(kwargs.get('lines', [])),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_info_line', **params)

        # Funny endpoint, no status code
        if not util.check_result(result, 'Line'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'Line')
        return True, [emtype.Line(**a) for a in values]

    def get_info_line_extended(self, **kwargs):
        """Obtain extended information on a bus line on a given date.

        Args:
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Line]), or message string
            in case of error.
        """
        # Endpoint parameters
        select_date = '%02d/%02d/%d' % (
            kwargs.get('day', '01'),
            kwargs.get('month', '01'),
            kwargs.get('year', '1970')
        )

        params = {
            'fecha': select_date,
            'line': util.ints_to_string(kwargs.get('lines', [])),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_info_line_extended', **params)

        # Funny endpoint, no status code
        if not util.check_result(result, 'Line'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'Line')
        return True, [emtype.Line(**a) for a in values]

    def get_poi(self, **kwargs):
        """Obtain a list of POI in the given radius.

        Args:
            latitude (double): Latitude in decimal degrees.
            longitude (double): Longitude in decimal degrees.
            types (list[int] | int): POI IDs (or empty list to get all).
            radius (int): Radius (in meters) of the search.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Poi]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'coordinateX': kwargs.get('longitude'),
            'coordinateY': kwargs.get('latitude'),
            'tipos': util.ints_to_string(kwargs.get('types')),
            'Radius': kwargs.get('radius'),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_poi', **params)

        # Funny endpoint, no status code
        if not util.check_result(result, 'poiList'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'poiList')
        return True, [emtype.Poi(**a) for a in values]

    def get_poi_types(self, **kwargs):
        """Obtain POI types.

        Args:
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[PoiType]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_poi_types', **params)

        # Parse
        values = result.get('types', [])
        return True, [emtype.PoiType(**a) for a in values]

    def get_route_lines_route(self, **kwargs):
        """Obtain itinerary for one or more lines in the given date.

        Args:
            day (int): Day of the month in format DD.
                The number is automatically padded if it only has one digit.
            month (int): Month number in format MM.
                The number is automatically padded if it only has one digit.
            year (int): Year number in format YYYY.
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.

        Returns:
            Status boolean and parsed response (list[RouteLinesItem]), or message
            string in case of error.
        """
        # Endpoint parameters
        select_date = '%02d/%02d/%d' % (
            kwargs.get('day', '01'),
            kwargs.get('month', '01'),
            kwargs.get('year', '1970')
        )

        params = {
            'SelectDate': select_date,
            'Lines': util.ints_to_string(kwargs.get('lines', []))
        }

        # Request
        result = self.make_request('geo', 'get_route_lines_route', **params)

        if not util.check_result(result):
            return False, result.get('resultDescription', 'UNKNOWN ERROR')

        # Parse
        values = util.response_list(result, 'resultValues')
        return True, [emtype.RouteLinesItem(**a) for a in values]

    def get_stops_from_stop(self, **kwargs):
        """Obtain a list of stops within the given radius of the specified stop.

        Args:
            stop_number (int): Number of the stop to query.
            radius (int): Radius (in meters) of the search.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Stop]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'idStop': kwargs.get('stop_number'),
            'Radius': kwargs.get('radius'),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_stops_from_stop', **params)

        # Funny endpoint, no status code
        if not util.check_result(result, 'stops'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'stops')
        return True, [emtype.Stop(**a) for a in values]

    def get_stops_from_xy(self, **kwargs):
        """Obtain a list of stops around the given point.

        Arguments:
            latitude (double): Latitude in decimal degrees.
            longitude (double): Longitude in decimal degrees.
            radius (int): Radius (in meters) of the search.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Stop]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'latitude': kwargs.get('latitude'),
            'longitude': kwargs.get('longitude'),
            'Radius': kwargs.get('radius'),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_stops_from_xy', **params)

        # Funny endpoint, no status code
        # No stop attribute could mean there are no stops in the zone specified
        if not util.check_result(result, 'stop'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'stop')
        return True, [emtype.Stop(**a) for a in values]

    def get_stops_line(self, **kwargs):
        """Obtain information on the stops of the given lines.

        Arguments:
            lines (list[int] | int): Lines to query, may be empty to get
                all the lines.
            direction (str): Optional, either *forward* or *backward*.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Stop]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'line': util.ints_to_string(kwargs.get('lines', [])),
            'direction': util.direction_code(kwargs.get('direction', '')),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_stops_line', **params)

        # Funny endpoint, no status code
        # Only interested in 'stop'
        if not util.check_result(result, 'stop'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'stop')
        return True, [emtype.Stop(**a) for a in values]

    def get_street(self, **kwargs):
        """Obtain a list of nodes related to a location within a given radius.

        Not sure of its use, but...

        Args:
            street_name (str): Name of the street to search.
            street_number (int): Street number to search.
            radius (int): Radius (in meters) of the search.
            stops (int): Number of the stop to search.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Site]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'description': kwargs.get('street_name'),
            'streetNumber': kwargs.get('street_number'),
            'Radius': kwargs.get('radius'),
            'Stops': kwargs.get('stops'),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_street', **params)

        # Funny endpoint, no status code
        if not util.check_result(result, 'site'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'site')
        return True, [emtype.Site(**a) for a in values]

    def get_street_from_xy(self, **kwargs):
        """Obtain a list of streets around the specified point.

        Args:
            latitude (double): Latitude in decimal degrees.
            longitude (double): Longitude in decimal degrees.
            radius (int): Radius (in meters) of the search.
            lang (str): Language code (*es* or *en*).

        Returns:
            Status boolean and parsed response (list[Street]), or message string
            in case of error.
        """
        # Endpoint parameters
        params = {
            'coordinateX': kwargs.get('longitude'),
            'coordinateY': kwargs.get('latitude'),
            'Radius': kwargs.get('radius'),
            'cultureInfo': util.language_code(kwargs.get('lang'))
        }

        # Request
        result = self.make_request('geo', 'get_street_from_xy', **params)

        # Funny endpoint, no status code
        if not util.check_result(result, 'site'):
            return False, 'UNKNOWN ERROR'

        # Parse
        values = util.response_list(result, 'site')
        return True, [emtype.Street(**a) for a in values]



# -*- coding: utf-8 -*-
# pyemtmad, EMT API wrapper - https://github.com/rmed/pyemtmad
# Copyright (C) 2016  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
This file contains some utility functions used in the API interface.
"""

import ssl
import six
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager


class ParkingAdapter(HTTPAdapter):
    """Custom HTTP adapter for parking API, as it uses TLSv1."""

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = PoolManager(num_pools=connections,
                                       maxsize=maxsize,
                                       block=block,
                                       ssl_version=ssl.PROTOCOL_TLSv1)


def check_result(data, key=''):
    """Check the result of an API response.

    Ideally, this should be done by checking that the value of the ``resultCode``
    attribute is 0, but there are endpoints that simply do not follow this rule.

    Args:
        data (dict): Response obtained from the API endpoint.
        key (string): Key to check for existence in the dict.

    Returns:
        bool: True if result was correct, False otherwise.
    """
    if not isinstance(data, dict):
        return False

    if key:
        if key in data:
            return True

        return False

    if 'resultCode' in data.keys():
        # OpenBus
        return True if data.get('resultCode', -1) == 0 else False

    elif 'code' in data.keys():
        # Parking
        return True if data.get('code', -1) == 0 else False

    return False

def date_string(day, month, year):
    """Build a date string using the provided day, month, year numbers.

    Automatically adds a leading zero to ``day`` and ``month`` if they only have
    one digit.

    Args:
        day (int): Day number.
        month(int): Month number.
        year(int): Year number.

    Returns:
        str: Date in the format *DD/MM/YYYY*.
    """
    return '%02d/%02d/%d' % (day, month, year)

def datetime_string(day, month, year, hour, minute):
    """Build a date string using the provided day, month, year numbers.

    Automatically adds a leading zero to ``day`` and ``month`` if they only have
    one digit.

    Args:
        day (int): Day number.
        month(int): Month number.
        year(int): Year number.
        hour (int): Hour of the day in 24h format.
        minute (int): Minute of the hour.

    Returns:
        str: Date in the format *YYYY-MM-DDThh:mm:ss*.
    """
    # Overflow
    if hour < 0 or hour > 23: hour = 0
    if minute < 0 or minute > 60: minute = 0

    return '%d-%02d-%02dT%02d:%02d:00' % (year, month, day, hour, minute)

def direction_code(direction):
    """Obtain the integer code of a direction string.

    Args:
        direction (str): *forward* or *backward*

    Returns:
        int: direction code
    """
    if direction == 'backward':
        return 2

    return 1

def ints_to_string(ints):
    """Convert a list of integers to a *|* separated string.

    Args:
        ints (list[int]|int): List of integer items to convert or single
            integer to convert.

    Returns:
        str: Formatted string
    """
    if not isinstance(ints, list):
        return six.u(str(ints))

    return '|'.join(six.u(str(l)) for l in ints)

def language_code(code):
    """Generate the ``cultureInfo`` language code for the API.

    Available codes are *EN* (default) and *ES*.

    Args:
        code (str): Two-character language code.

    Returns:
        str: Language code in the format accepted by the API.
    """
    if not code or code == 'es':
        return 'ES'

    return 'EN'

def response_list(data, key):
    """Obtain the relevant response data in a list.

    If the response does not already contain the result in a list, a new one
    will be created to ease iteration in the parser methods.

    Args:
        data (dict): API response.
        key (str): Attribute of the response that contains the result values.

    Returns:
        List of response items (usually dict) or None if the key is not present.
    """
    if key not in data:
        return None

    if isinstance(data[key], list):
        return data[key]

    else:
        return [data[key],]



# -*- coding: utf-8 -*-
# pyemtmad, EMT API wrapper - https://github.com/rmed/pyemtmad
# Copyright (C) 2016  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""This file contains the API URL and endpoints for EMT.

See http://opendata.emtmadrid.es/Servicios-web
"""

import requests
from pyemtmad.api.bus import BusApi
from pyemtmad.api.geo import GeoApi
from pyemtmad.api.parking import ParkingApi
from pyemtmad.util import ParkingAdapter

# Custom request for parking API
_parking_req = requests.Session()
_parking_req.mount('https://', ParkingAdapter())

# API URLs
URL_OPENBUS = 'https://openbus.emtmadrid.es:9443/emt-proxy-server/last/'
URL_PARKING = 'https://servicios.emtmadrid.es:8443/infoParking/infoParking.svc/json/'


# JSON endpoints
ENDPOINTS_BUS = {
    'get_calendar'           : 'bus/GetCalendar.php',
    'get_groups'             : 'bus/GetGroups.php',
    'get_list_lines'         : 'bus/GetListLines.php',
    'get_nodes_lines'        : 'bus/GetNodesLines.php',
    'get_route_lines'        : 'bus/GetRouteLines.php',
    'get_route_lines_route'  : 'bus/GetRouteLinesRoute.php',
    'get_times_lines'        : 'bus/GetTimesLines.php',
    'get_timetable_lines'    : 'bus/GetTimeTableLines.php'
}

ENDPOINTS_GEO = {
    'get_arrive_stop'        : 'geo/GetArriveStop.php',
    'get_arrive_client'      : 'geo/GetArriveClient.php',
    'get_groups'             : 'geo/GetGroups.php',
    'get_info_line'          : 'geo/GetInfoLine.php',
    'get_info_line_extended' : 'geo/GetInfoLineExtend.php',
    'get_poi'                : 'geo/GetPointsOfInterest.php',
    'get_poi_types'          : 'geo/GetPointsOfInterestTypes.php',
    'get_route_lines_route'  : 'geo/GetRouteLinesRoute.php',
    'get_stops_from_stop'    : 'geo/GetStopsFromStop.php',
    'get_stops_from_xy'      : 'geo/GetStopsFromXY.php',
    'get_stops_line'         : 'geo/GetStopsLine.php',
    'get_street'             : 'geo/GetStreet.php',
    'get_street_from_xy'     : 'geo/GetStreetFromXY.php'
}

ENDPOINTS_PARKING = {
    'detail_parking'          : 'detailParking/{id_client},{passkey}',
    'detail_poi'              : 'detailPOI/{id_client},{passkey}',
    'icon_description'        : 'iconDescription/{id_client},{passkey}',
    'info_parking_poi'        : 'infoParkingPoi/{id_client},{passkey}',
    'list_features'           : 'listFeatures/{id_client},{passkey}',
    'list_parking'            : 'listFeatures/{id_client},{passkey},{lang}',
    'list_street_poi_parking' : 'listStreetPoisParking/{id_client},{passkey},{address},{lang}',
    'list_types_poi'          : 'listTypesPOIs/{id_client},{passkey},{lang}'
}


class Wrapper(object):
    """Interface for the JSON API of the EMT services."""

    def __init__(self, emt_id='', emt_pass=''):
        """Initialize the interface attributes.

        Initialization may also be performed at a later point by manually
        calling the ``initialize()`` method.

        Args:
            emt_id (str): ID given by the server upon registration
            emt_pass (str): Token given by the server upon registration
        """
        if emt_id and emt_pass:
            self.initialize(emt_id, emt_pass)

    def initialize(self, emt_id, emt_pass):
        """Manual initialization of the interface attributes.

        This is useful when the interface must be declare but initialized later
        on with parsed configuration values.

        Args:
            emt_id (str): ID given by the server upon registration
            emt_pass (str): Token given by the server upon registration
        """
        self._emt_id = emt_id
        self._emt_pass = emt_pass

        # Initialize modules
        self.bus = BusApi(self)
        self.geo = GeoApi(self)
        self.parking = ParkingApi(self)

    def request_openbus(self, service, endpoint, **kwargs):
        """Make a request to the given endpoint of the ``openbus`` server.

        This returns the plain JSON (dict) response which can then be parsed
        using one of the implemented types.

        Args:
            service (str): Service to fetch ('bus' or 'geo').
            endpoint (str): Endpoint to send the request to.
                This string corresponds to the key in the ``ENDPOINTS`` dict.
            **kwargs: Request arguments.

        Returns:
            Obtained response (dict) or None if the endpoint was not found.
        """
        if service == 'bus':
            endpoints = ENDPOINTS_BUS

        elif service == 'geo':
            endpoints = ENDPOINTS_GEO

        else:
            # Unknown service
            return None

        if endpoint not in endpoints:
            # Unknown endpoint
            return None

        url = URL_OPENBUS + endpoints[endpoint]

        # Append credentials to request
        kwargs['idClient'] = self._emt_id
        kwargs['passKey'] = self._emt_pass

        # SSL verification fails...
        # return requests.post(url, data=kwargs, verify=False).json()
        return requests.post(url, data=kwargs, verify=True).json()

    def request_parking(self, endpoint, url_args={}, **kwargs):
        """Make a request to the given endpoint of the ``parking`` server.

        This returns the plain JSON (dict) response which can then be parsed
        using one of the implemented types.

        Args:
            endpoint (str): Endpoint to send the request to.
                This string corresponds to the key in the ``ENDPOINTS`` dict.
            url_args (dict): Dictionary for URL string replacements.
            **kwargs: Request arguments.

        Returns:
            Obtained response (dict) or None if the endpoint was not found.
        """
        if endpoint not in ENDPOINTS_PARKING:
            # Unknown endpoint
            return None

        url = URL_OPENBUS + ENDPOINTS_PARKING[endpoint]

        # Append additional info to URL
        lang = url_args.get('lang', 'ES')
        address = url_args.get('address', '')

        url = url.format(
            id_client=self._emt_id,
            passkey=self._emt_pass,
            address=address,
            language=lang
        )

        # This server uses TLSv1
        return _parking_req.post(url, data=kwargs).json()



# -*- coding: utf-8 -*-
# pyemtmad, EMT API wrapper - https://github.com/rmed/pyemtmad
# Copyright (C) 2016  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""This file contains type definitions for the data returned by the API."""

import datetime

WEEK_DAYS = {
    'L': 'Monday',
    'M': 'Tuesday',
    'X': 'Wednesday',
    'J': 'Thursday',
    'V': 'Friday',
    'S': 'Saturday',
    'D': 'Sunday'
}

DAY_TYPES = {
    'LA'        : 'Labour',
    'V'         : 'Friday',
    'SA'        : 'Saturday',
    'FE'        : 'Festive',
    'LABORABLE' : 'Labour',
    'SABADO'    : 'Saturday',
    'FESTIVO'   : 'Festive'
}

SEC_DETAILS = {
    10: 'forward_stop',
    19: 'forward_vertex',
    20: 'backward_stop',
    29: 'backward_vertex'
}

POI_TYPES = {
    1 : 'cars',
    10: 'shopping',
    11: 'tourism',
    12: 'metro',
    13: 'light rail',
    14: 'interchange',
    15: 'transport places I',
    16: 'transport places II',
    17: 'RENFE',
    2 : 'services',
    3 : 'education',
    4 : 'leisure',
    5 : 'financial entities',
    6 : 'hospitals',
    7 : 'parkings',
    8 : 'recreation centers',
    9 : 'restaurants'
}

PARKING_POI_TYPES = {
    'Zona'    : 'zone',
    'Poi'     : 'poi',
    'Parking' : 'parking'
}


class Arrival(object):
    """
    Attributes:
        stop_id (int): Stop ID.
        line_id (string): Line ID.
        is_header (bool): Whether or not the stop is header of the line.
        destination (string): Destination of the bus.
        bus_id (int): Bus ID.
        time_left (int): Seconds remaining until bus arrives.
            If this value is >= 999999, then 20+ minutes remain.
        distance (int): distance of the bus in meters.
        longitude (double): Longitude of the bus in decimal degrees
        latitude (double): Latitude of the bus in decimal degrees
        position_type (string): Real or estimate position

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.stop_id = kwargs.get('stopId')
        self.line_id = kwargs.get('lineId')
        self.is_head = False if kwargs.get('isHead') == 'False' else True
        self.destination = kwargs.get('destination')
        self.bus_id = int(kwargs.get('busId', '-1'))
        self.time_left = kwargs.get('busTimeLeft')
        self.distance = kwargs.get('busDistance')
        self.longitude = kwargs.get('longitude')
        self.latitude = kwargs.get('latitude')

        pos_type = kwargs.get('busPositionType')
        if pos_type == 0:
            self.position_type = 'real'

        elif pos_type == 1:
            self.position_type = 'estimate'


        self._json = kwargs


class BusGroupItem(object):
    """
    Attributes:
        id (int): Group ID.
        description (string): Group description.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = int(kwargs.get('groupId'))
        self.description = kwargs.get('groupDescription').strip()

        self._json = kwargs


class CalendarItem(object):
    """
    Attributes:
        date (string): Date in DD/MM/YYYY H:mm:ss format.
        day_type (string): Day type. May be:
            - labour (LA)
            - friday (F)
            - saturday (SA)
            - festive (FE)
        strike (boolean): Whether there is strike on this day or not.
        day_of_week (string): Day of the week (in english) obtained from
            single character Spanish representation.
        week (int): Week number.
        month (int): Month number.
        trimester (int): Trimester number.
        quarter (int): Quarter number.
        semester (int): Semester number.
        year (int): Year number.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.date = kwargs.get('date', '').replace('\\', '').strip()
        self.week = kwargs.get('week')
        self.month = kwargs.get('month')
        self.trimester = kwargs.get('trimester')
        self.quarter = kwargs.get('quarter')
        self.semester = kwargs.get('semester')
        self.year = kwargs.get('year')

        self.day_type = DAY_TYPES.get(kwargs.get('typeDay'), 'Labour')

        on_strike = kwargs.get('strike', 'N')
        if on_strike == 'Y':
            self.strike = True

        elif on_strike == 'N':
            self.strike = False

        day_week = kwargs.get('dayOfWeek')
        self.day_of_week = WEEK_DAYS.get(day_week, 'Monday')

        self._json = kwargs


class DayType(object):
    """
    Attributes:
        day_type (string): Day type. May be:
            - labour (LA)
            - friday (F)
            - saturday (SA)
            - festive (FE)
        dir_forward (Direction): Forward line direction details.
        dir_backward (Direction): Backward line direction details.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.day_type = DAY_TYPES.get(kwargs.get('dayTypeId'), 'Labour')
        self.dir_forward = Direction(**kwargs.get('direction1'))
        self.dir_backward = Direction(**kwargs.get('direction2'))

        self._json = kwargs


class Direction(object):
    """
    Attributes:
        start_time (string): Starting time in HH:mm.
        end_time (string): Ending time in HH:mm.
        minimum_frequency (int): Minimum frequency of buses in minutes.
        maximum_frequency (int): Maximum frequency of buses in minutes.
        frequency_description (string): Human readable description of the
            frequency (in extended call).

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.start_time = kwargs.get('startTime')
        self.end_time = kwargs.get('endTime')
        self.minimum_frequency = kwargs.get('minimumFrequency')
        self.maximum_frequency = kwargs.get('maximumFrequency')
        self.frequency_description = kwargs.get('frequencyDescription', '') \
                .replace('\\', '').strip()

        self._json = kwargs


class GeoGroupItem(object):
    """
    Attributes:
        id (int): group ID.
        subgroup (int): subgroup ID.
        start_date (int): starting date timestamp.
        end_date (int): ending date timestamp.
        description (string): description text.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get('groupId')
        self.subgroup = kwargs.get('subGroupId')
        self.start_date = kwargs.get('startDate')
        self.end_date = kwargs.get('endDate')
        self.description = kwargs.get('description')

        self._json = kwargs


class IconDescription(object):
    """
    Attributes:
        classification (str): Localized name of the group of the icon.
        classification_spanish (str): Original name of the group of the icon.
        description (str): Description of the referenced element.
        icon_name (str): Name of the element.
        url_icon (str): URL to the icon.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.classification = kwargs.get('classificationTranslated')
        self.classification_spanish = kwargs.get('classification')
        self.description = kwargs.get('description')
        self.icon_name = kwargs.get('iconName')
        self.url_icon = kwargs.get('urlIcon')

        self._json = kwargs


class InfoParkingPoi(object):
    """
    Attributes:
        id (int): Parking or POI ID. They are identified with (family, id).
        name (str): Parking or POI name.
        address (str): Parking or POI address.
        type (str): Parking or POI type.
        type_code (str): Parking or POI type code.
        administrative_area (str): Administrative area in which the parking or
            POI is located. This is usually 'Madrid'.
        area_code (int): Postal code of the parking or POI.
        category (str): Parking or POI category name.
        category_code(str): Parking or POI category code.
        country (str): Country in which the parking or POI is located ('Spain').
        email (str): Public email of the POI.
        family (str): Family of the parking or POI.
        family_code (str): Unique code of the family.
        fax (str): Public fax of the POI.
        url_icon (str): URL to the icon representing the parking or POI.
        state (str): State in which the parking or POI is located.
        telephone (str): Public telephone of the POI.
        town (str): Town in which the parking or POI is located.
        latitude (double): Latitude of the parking or POI in decimal degrees.
        longitude (double): Longitude of the parking or POI in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.address = kwargs.get('address')
        self.type = kwargs.get('type')
        self.type_code = kwargs.get('typeCode')
        self.administrative_area = kwargs.get('administrativeArea')
        self.area_code = int(kwargs.get('areaCode', '-1'))
        self.category = kwargs.get('category')
        self.category_code = kwargs.get('categoryCode')
        self.country = kwargs.get('country')
        self.email = kwargs.get('email')
        self.family = kwargs.get('familyName')
        self.family_code = kwargs.get('family')
        self.fax = kwargs.get('fax')
        self.url_icon = kwargs.get('icon')
        self.state = kwargs.get('state')
        self.telephone = kwargs.get('telephone')
        self.town = kwargs.get('town')
        self.latitude = float(kwargs.get('latitude', '0.0'))
        self.longitude = float(kwargs.get('longitude', '0.0'))

        self._json = kwargs

class Line(object):
    """
    Attributes:
        id (int): Line ID.
        label (string): Line label.
        date (string): Date in which information was updated (dd/mm/YYYY).
        header_a (string): Name of the end of the line A.
        header_b (string): Name of the end of the line B.
        incidents (int): Number of incidents in the line.
        day_types (list[DayType]) - list of day types for this line.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get('lineId')
        self.label = kwargs.get('label')
        self.header_a = kwargs.get('headerA')
        self.header_b = kwargs.get('headerB')
        self.incidents = kwargs.get('incidents')

        # Extended version returns a timestamp
        if 'date' in kwargs.keys():
            # Extended
            self.date = datetime.datetime.fromtimestamp(
                kwargs.get('date', 0) / 1e3)

        elif 'string' in kwargs.keys():
            # Basic
            self.date = kwargs.get('string', '').replace('\\', '').strip()

        self.day_types = [DayType(**a) for a in kwargs.get('dayType', [])]

        self._json = kwargs


class LineInfo(object):
    """
    Attributes:
        day_type (string): Day type. May be:
            - labour (LA)
            - friday (F)
            - saturday (SA)
            - festive (FE)
        line (int): Line number.
        label (string): Label of the line.
        header_a (string): Name of the end of the line A.
        header_b (string): Name of the end of the line B.
        direction (string): Direction of the line.
        start_time (string): Time in which the line starts service (HH:mm).
        end_time (string): Time in which the line ends service (HH:mm).
        minimum_frequency (int): minimum frequency of buses in minutes.
        maximum_frequency (int):maximum frequency of buses in minutes.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.line = int(kwargs.get('name'))
        self.label = kwargs.get('label')
        self.header_a = kwargs.get('headerA')
        self.header_b = kwargs.get('headerB')
        self.start_time = kwargs.get('startTime')
        self.end_time = kwargs.get('stopTime')
        self.minimum_frequency = int(kwargs.get('minimumFrequency', '-1'))
        self.maximum_frequency = int(kwargs.get('maximumFrequency', '-1'))

        self.day_type = DAY_TYPES.get(kwargs.get('typeDay'), 'Labour')

        direction = kwargs.get('direction', 'A')
        if direction == 'A':
            self.direction = 'forward'
        elif direction == 'B':
            self.direction = 'backward'

        self._json = kwargs


class ListLineInfo(object):
    """
    Attributes:
        group (int): Group number.
        start_date (string): Date in which line started service (DD/MM/YYYY).
        end_date (string): Date in which line ended service (DD/MM/YYYY).
        line (int): Line number.
        label (string): Label of the line.
        header_a (string): Name of the end of the line A.
        header_b (string): Name of the end of the line B.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.group = int(kwargs.get('groupNumber', '0'))
        self.start_date = kwargs.get('dateFirst', '').replace('\\', '').strip()
        self.end_date = kwargs.get('dateEnd', '').replace('\\', '').strip()
        self.line = int(kwargs.get('line', '0'))
        self.label = kwargs.get('label')
        self.header_a = kwargs.get('nameA')
        self.header_b = kwargs.get('nameB')

        self._json = kwargs


class NodeLinesItem(object):
    """
    Attributes:

        id (int): Node ID.
        name (string): Node name.
        lines (list[tuple]): Line ID and direction.
        latitude (double): Latitude of the node in decimal degrees.
        longitude (double): Longitude of the node in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get('node')
        self.name = kwargs.get('name')
        self.latitude = kwargs.get('latitude')
        self.longitude = kwargs.get('longitude')

        lines = kwargs.get('lines')
        self.lines = []

        for line in lines:
            elements = line.replace('\\', '').strip().split('/')

            # 1 = forward, 2 = backward
            direction = 'forward' if elements[1] == '1' else 'backward'
            self.lines.append((int(elements[0]), direction))

        self._json = kwargs


class Parking(object):
    """
    Attributes:
        id (int): Parking ID. Parkings are identified with (family, id).
        name (str): Parking name.
        address (str): Parking address.
        type (str): Parking type.
        administrative_area (str): Administrative area in which the parking is
            located. This is usually 'Madrid'.
        area_code (int): Postal code of the parking.
        category (str): Parking category code.
        country (str): Country in which the parking is located ('Spain').
        family (str): Family of the parking.
        family_code (str): Unique code of the family.
        nickname (str): Nickname of the parking.
        state (str): State in which the parking is located.
        town (str): Town in which the parking is located.
        latitude (double): Latitude of the parking in decimal degrees.
        longitude (double): Longitude of the parking in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.address = kwargs.get('address')
        self.type = kwargs.get('type')
        self.administrative_area = kwargs.get('administrativeArea')
        self.area_code = int(kwargs.get('areaCode', '-1'))
        self.category = kwargs.get('category')
        self.country = kwargs.get('country')
        self.family = kwargs.get('family')
        self.family_code = kwargs.get('familyCode')
        self.nickname = kwargs.get('nickName')
        self.state = kwargs.get('state')
        self.town = kwargs.get('town')
        self.latitude = float(kwargs.get('latitude', '0.0'))
        self.longitude = float(kwargs.get('longitude', '0.0'))

        self._json = kwargs


class ParkingAccess(object):
    """
    Attributes:
        name (str): Name of the access.
        address (str): Address where the access is located.
        code (str): ID code of the access.
        url_icon (str): URL to the icon representing the access.
        latitude (double): Latitude of the access in decimal degrees.
        longitude (double): Longitude of the access in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.name = kwargs.get('name')
        self.address = kwargs.get('address')
        self.code = kwargs.get('code')
        self.url_icon = kwargs.get('url_icon')
        self.latitude = float(kwargs.get('latitude', '0.0'))
        self.longitude = float(kwargs.get('longitude', '0.0'))

        self._json = kwargs

class ParkingDetails(object):
    """
    Attributes:
        id (int): ID of the parking.
        name (str): Name of the parking.
        schedule (str): Time in which the parking is open.
        address (str): Parking address.
        administrative_area (str): Administrative area in which the parking is
            located. This is usually 'Madrid'.
        area_code (str): Postal code of the parking.
        category (str): Name of the parking category.
        category_code (str): Parking category code.
        type (str): Parking type.
        type_code (str): Parking type code.
        country (str): Country in which the parking is located ('Spain').
        url_icon (str): URL to the icon representing this feature, if any.
        nickname (str): Nickname of the parking.
        state (str): State in which the parking is located.
        town (str): Town in which the parking is located.
        latitude (double): Latitude of the parking in decimal degrees.
        longitude (double): Longitude of the parking in decimal degrees.

        accesses (list[ParkingAccess]): List of accesses to this parking.
        features (list[ParkingFeature]): List of features of this parking.
        occupation (list[ParkingOccupation]): List of occupation details.
        rates (list[ParkingRate]): List of parking rates.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        general = kwargs.get('general', {})

        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.schedule = kwargs.get('schedule')
        self.address = general.get('address')
        self.administrative_area = general.get('administrativeArea')
        self.area_code = general.get('areaCode')
        self.category = general.get('category')
        self.category_code = general.get('categoryCode')
        self.type = general.get('type')
        self.type_code = general.get('typeCode')
        self.country = general.get('country')
        self.family = kwargs.get('family')
        self.family_code = kwargs.get('familyCode')
        self.nickname = general.get('nickName')
        self.state = general.get('state')
        self.town = general.get('town')
        self.latitude = float(general.get('latitude', '0.0'))
        self.longitude = float(general.get('longitude', '0.0'))

        accesses = kwargs.get('lstAccess', [])
        features = kwargs.get('lstFeatures', [])
        occupation = kwargs.get('lstOccupation', [])
        rates = kwargs.get('lstRates', [])

        self.accesses = [ParkingAccess(**a) for a in accesses]
        self.features = [ParkingFeature(**a) for a in features]
        self.occupation = [ParkingOccupation(**a) for a in occupation]
        self.rates = [ParkingRate(**a) for a in rates]

        self._json = kwargs


class ParkingFeature(object):
    """
    Attributes:
        name (str): Name of the feature.
        code (str): 3-char ID code of the feature.
        content (str): Content of the feature, if any.
        description (str): Description of the feature, if any.
        field (str): Localized name of the group the feature belongs to.
        field_spanish (str): Original name of the group the feature belongs to.
        url_icon (str): URL to the icon representing this feature, if any.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.name = kwargs.get('name')
        self.code = kwargs.get('nameCode')
        self.description = kwargs.get('description')
        self.field = kwargs.get('nameFieldTranslated')
        self.field_spanish = kwargs.get('nameField')
        self.url_icon = kwargs.get('urlIcon')

        self._json = kwargs


class ParkingOccupation(object):
    """
    Attributes:
        type (str): Type of slot.
        code (str): 3-char ID code of the feature.
        free (int): Number of free slots.
        moment (str): Moment in which the data was updated.
        renewal_index (double): Renewal index of the parking.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.type = kwargs.get('name')
        self.code = kwargs.get('code')
        self.free = kwargs.get('free')
        self.moment = kwargs.get('moment')
        self.renewal_index = kwargs.get('renewal_index')

        self._json = kwargs


class ParkingPoi(object):
    """
    Attributes:
        id (int): Parking ID. Parkings are identified with (family, id).
        name (str): Parking name.
        address (str): Parking address.
        address_number (str): Street number of the POI (only for addresses).
        type (str): Element type. Possible types are:
            - zone
            - poi
            - parking
        administrative_area (str): Administrative area in which the parking is
            located. This is usually 'Madrid'.
        area_code (int): Postal code of the parking.
        category (str): Parking category code.
        country (str): Country in which the parking is located ('Spain').
        email (str): Email of the POI (if any).
        family (str): Family of the parking.
        family_code (str): Unique code of the family.
        fax (str): Fax number of the POI (if any).
        url_icon (str): URL to the icon representing this POI, if any.
        nickname (str): Nickname of the parking.
        state (str): State in which the parking is located.
        telephone (str): Phone number.
        town (str): Town in which the parking is located.
        latitude (double): Latitude of the parking in decimal degrees.
        longitude (double): Longitude of the parking in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.address = kwargs.get('address')
        self.address_number = kwargs.get('addressNumber')
        self.type = PARKING_POI_TYPES.get(kwargs.get('type', ''), 'zone')
        self.administrative_area = kwargs.get('administrativeArea')
        self.area_code = int(kwargs.get('areaCode', '-1'))
        self.category = kwargs.get('category')
        self.country = kwargs.get('country')
        self.email = kwargs.get('email')
        self.family = kwargs.get('family')
        self.family_code = kwargs.get('familyCode')
        self.fax = kwargs.get('fax')
        self.url_icon = kwargs.get('neutralIcon')
        self.nickname = kwargs.get('nickName')
        self.state = kwargs.get('state')
        self.telephone = kwargs.get('telephone')
        self.town = kwargs.get('town')
        self.latitude = float(kwargs.get('latitude', '0.0'))
        self.longitude = float(kwargs.get('longitude', '0.0'))

        self._json = kwargs


class ParkingPoiType(object):
    """
    Attributes:
        name (str): Name of the POI type.
        code (str): Code of the POI type.
        category (str): Name of the POI category.
        category_code (str): Code of the POI category.
        family (str): Name of the POI family.
        family_code (str): Unique code of the family.
        url_icon (str): URL to the icon representing this POI, if any.

        _json (dict): Original API response.
    """
    def __init__(self, **kwargs):
        self.name = kwargs.get('type')
        self.code = kwargs.get('typeCode')
        self.category = kwargs.get('category')
        self.category_code = kwargs.get('categoryCode')
        self.family = kwargs.get('family')
        self.family_code = kwargs.get('familyCode')
        self.url_icon = kwargs.get('neutralIcon')

        self._json = kwargs


class ParkingRate(object):
    """
    Attributes:
        description (str): Rate description.
        start_minutes (int): Start minutes of the stay.
        end_minutes (int): End minutes of the stay.
        type (str): Type of rate (minutes, max, etc.)
        rate (double): Amount to pay given the rate type
        schedule_start (str): Starting time of the rate.
        schedule_end (str): End time of the rate.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.description = kwargs.get('description')
        self.start_minutes = kwargs.get('minutesStayInitiation')
        self.end_minutes = kwargs.get('minutesStayEnd')
        self.type = kwargs.get('periodPricing')
        self.rate = kwargs.get('rate')
        self.schedule_start = kwargs.get('scheduleInitial')
        self.schedule_end = kwargs.get('scheduleEnd')

        self._json = kwargs


class Poi(object):
    """
    Attributes:
        id (int): POI ID.
        poi_type (string): POI type.
        name (string): POI name.
        address (string): POI address.
        street_number (int): Street number of the POI.
        phone_number (string): Phone number of the POI (if any).
        latitude (double): latitude in decimal degrees.
        longitude (double): longitude in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        attrs = kwargs.get('attributes', {})
        self.id = attrs.get('poiId')
        self.poi_type = POI_TYPES.get(attrs.get('poiType', -1), '')
        self.name = attrs.get('name')
        self.address = attrs.get('address')
        self.street_number = attrs.get('streetNumber')
        self.phone_number = attrs.get('phoneNumber')
        self.latitude = attrs.get('latitude')
        self.longitude = attrs.get('longitude')

        self._json = kwargs


class PoiDetails(object):
    """
    Attributes:
        id (int): ID of the POI.
        name (str): Localized name of the POI.
        name_spanish (str): Original POI name.
        description (str): Description of the POI.
        schedule (str): Time in which the POI is open.
        address (str): Parking address.
        administrative_area (str): Administrative area in which the POI is
            located. This is usually 'Madrid'.
        area_code (str): Postal code of the POI.
        category (str): Name of the POI category.
        category_code (str): POI category code.
        type (str): POI type.
        type_code (str): POI type code.
        country (str): Country in which the parking is located ('Spain').
        email (str): Email of the POI, if any.
        telephone (str): Public phone number of the POI, if any.
        fax (str): Public fax of the POI, if any.
        url_icon (str): URL to the icon representing this feature, if any.
        state (str): State in which the parking is located.
        town (str): Town in which the parking is located.
        services_payment (str): Payment services.
        web (str): Website of the POI, if any.
        latitude (double): Latitude of the parking in decimal degrees.
        longitude (double): Longitude of the parking in decimal degrees.

        images (list[PoiImage]): List of images attached to this POI.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        info = kwargs.get('poiDetailInfo', {})

        self.id = kwargs.get('id')
        self.name = info.get('nameTranslated')
        self.name_spanish = kwargs.get('name')
        self.description = info.get('description')
        self.schedule = info.get('schedule')
        self.address = kwargs.get('address')
        self.administrative_area = kwargs.get('administrativeArea')
        self.area_code = kwargs.get('areaCode')
        self.category = kwargs.get('category')
        self.category_code = kwargs.get('categoryCode')
        self.type = kwargs.get('type')
        self.type_code = kwargs.get('typeCode')
        self.country = kwargs.get('country')
        self.email = kwargs.get('email')
        self.telephone = kwargs.get('telephone')
        self.fax = kwargs.get('fax')
        self.url_icon = kwargs.get('icon')
        self.state = kwargs.get('state')
        self.town = kwargs.get('town')
        self.services_payment = info.get('servicesPayment')
        self.web = info.get('web')
        self.latitude = kwargs.get('latitude')
        self.longitude = kwargs.get('longitude')

        detail_images = kwargs.get('poiDetailImages', [])
        self.images = [PoiImage(**a) for a in detail_images]

        self._json = kwargs


class PoiImage(object):
    """
    Attributes:
        description (str): Description of the image, if any.
        url (str): URL to the image.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.description = kwargs.get('description')
        self.url = kwargs.get('urlImage')

        self._json = kwargs


class PoiType(object):
    """
    Attributes:
        id (int): Type ID.
        name (string): Type name.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        attrs = kwargs.get('attributes', {})

        self.id = attrs.get('type')
        self.name = attrs.get('name')

        self._json = kwargs


class RouteLinesItem(object):
    """
    Attributes:
        id (int): Node ID.
        line (int): Line to which the node belongs.
        node_type (string): Node type. May be:
            - forward_stop (10)
            - forward_vertex (19)
            - backward_stop (20)
            - backward_vertex (29)
        distance_orig (double): Distance to origin in meters.
        distance_prev (double): Distance to previous stop in meters.
        name (string): Name of the stop.
        latitude (double): Latitude in decimal degrees.
        longitude (double): Longitude in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get('node')
        self.line = kwargs.get('line')
        self.distance_orig = kwargs.get('distance')
        self.distance_prev = kwargs.get('distancePreviousStop')
        self.name = kwargs.get('name')
        self.latitude = kwargs.get('latitude')
        self.longitude = kwargs.get('longitude')

        sec_detail = kwargs.get('secDetail')
        self.node_type = SEC_DETAILS.get(sec_detail, 'forward_stop')

        self._json = kwargs


class Site(object):
    """
    Attributes:
        id (int): Site ID.
        site_type (string): Site type. May be:
            - street
            - poi
        name (string): Name of the site.
        longitude (double): longitude of the site in decimal degrees.
        latitude (double): latitude of the site in decimal degrees.

        street_type (string): Type of the street (avenue, street, etc.).
        number_type (string): Type of street numbering.
        street_number (int): Street number.
        zip_code (int): Street zip code.

        poi_type (string): Type of POI.
        poi_address (string): Address of the POI.
        poi_street_number (int): Street number of the POI.
        poi_direction (string): Direction of the POI.
        poi_phone_number (string): Phone number of the POI.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = int(kwargs.get('siteId', '0'))
        self.name = kwargs.get('description')
        self.longitude = kwargs.get('longitude')
        self.latitude = kwargs.get('latitude')

        site_type = kwargs.get('site_type', '')
        if site_type == 'C':
            self.site_type = 'street'
        elif site_type == 'P':
            self.site_type = 'poi'

        # Street
        self.street_type = kwargs.get('streetType')
        self.number_type = kwargs.get('numberType')
        self.street_number = int(kwargs.get('streetNumber', '-1'))
        self.zip_code = int(kwargs.get('zipCode', '-1'))

        # POI
        self.poi_type = POI_TYPES.get(int(kwargs.get('poiType', '-1')), '')
        self.poi_address = kwargs.get('poiAddress')
        self.poi_street_number = int(kwargs.get('poiStreetNumber', '-1'))
        self.poi_direction = kwargs.get('poiDirection')
        self.poi_phone_number = kwargs.get('poiPhoneNumber')

        self._json = kwargs


class Stop(object):
    """
    Attributes:
        id (int): Stop ID.
        name (string): Stop name.
        address (string): Postal address of the stop.
        longitude (double): Longitude of the stop in decimal degrees.
        latitude (double): Latitude of the stop in decimal degrees.
        lines (list[LineInfo]): Lines that stop here.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.id = int(kwargs.get('stopId'))
        self.name = kwargs.get('name')
        self.address = kwargs.get('postalAddress')
        self.longitude = kwargs.get('longitude')
        self.latitude = kwargs.get('latitude')

        if 'line' in kwargs:
            if isinstance(kwargs.get('line'), list):
                # Several lines
                self.lines = [LineInfo(**a) for a in kwargs.get('line')]

            else:
                # Single line
                self.lines = [LineInfo(**kwargs.get('line'))]

        self._json = kwargs


class Street(object):
    """
    Attributes:
        name (string): Name of the street.
        street_type (string): Type of street.
        street_number (int): Number of the street building.
        latitude (double): Latitude in decimal degrees.
        longitude (double): Longitude in decimal degrees.

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        attrs = kwargs.get('attributes', {})
        self.name = attrs.get('streetName')
        self.street_type = attrs.get('streetType')
        self.street_number = int(attrs.get('number', '-1'))
        self.latitude = attrs.get('latitude')
        self.longitude = attrs.get('longitude')

        self._json = kwargs


class TimesLinesItem(object):
    """
    Attributes:
        start_date (string): Date in which line started service (DD/MM/YYYY).
        end_date (string): Date in which line ended service (DD/MM/YYYY).
        line (int): Line number.
        day_type (string): Day type. May be:
            - labour (LA)
            - friday (F)
            - saturday (SA)
            - festive (FE)
        first_forward (string): Date and time of first bus forward (A-B).
            (DD/MM/YYYY H:mm:ss)
        first_backward (string): Date and time of first bus backward (B-A).
            (DD/MM/YYYY H:mm:ss)
        last_forward (string): Date and time of last bus forward (A-B).
            (DD/MM/YYYY H:mm:ss)
        last_backward (string): Date and time of last bus backward (B-A).
            (DD/MM/YYYY H:mm:ss)

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.start_date = kwargs.get('dateFirst', '').replace('\\', '').strip()
        self.end_date = kwargs.get('dateEnd', '').replace('\\', '').strip()
        self.line = int(kwargs.get('line', '0'))
        self.first_forward = kwargs.get('timeFirstA').replace('\\', '').strip()
        self.first_backward = kwargs.get('timeFirstB').replace('\\', '').strip()
        self.last_forward = kwargs.get('timeEndA').replace('\\', '').strip()
        self.last_backward = kwargs.get('timeEndB').replace('\\', '').strip()

        self.day_type = DAY_TYPES.get(kwargs.get('typeId'), 'Labour')

        self._json = kwargs


class TimetableLinesItem(object):
    """
    Attributes:
        date (string): Date for the timetable (DD/MM/YYYY).
        day_type (string): Day type. May be:
            - labour (LA)
            - friday (F)
            - saturday (SA)
            - festive (FE)
        line (int): Line number
        direction (string) - forward or backward
        trip (int) - trip number.
        start_time (string): Starting time (HH:mm:ss).
        end_time (string): Ending time (HH:mm:ss).

        _json (dict): Original API response.
    """

    def __init__(self, **kwargs):
        self.date = kwargs.get('date', '').replace('\\', '').strip()
        self.line = int(kwargs.get('line', '0'))
        self.start_time = kwargs.get('timeFirst', '').replace('\\', '').strip()
        self.end_time = kwargs.get('timeEnd', '').replace('\\', '').strip()

        self.day_type = DAY_TYPES.get(kwargs.get('typeDay'), 'Labour')

        direction = int(kwargs.get('direction'))
        if direction == 1:
            self.direction = 'forward'

        elif direction == 2:
            self.direction = 'backward'

        self._json = kwargs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pyemtmad, EMT API wrapper - https://github.com/rmed/pyemtmad
# Copyright (C) 2016  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "DESCRIPTION.rst")) as f:
    long_description = f.read()


setup(
        name='pyemtmad',
        version='0.1.0',

        description='(Kind of) sane wrapper for the EMT API',
        long_description=long_description,

        author='Rafael Medina García',
        author_email='rafamedgar@gmail.com',
        url='https://github.com/rmed/pyemtmad',

        license='GPLv2+',

        packages=find_packages(),
        install_requires=[
            'six >= 1.10.0',
            'requests >= 2.9.1'
            ],

        keywords='madrid transport travel bus geo open data api'
        )
